(function () {
    'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */

    function __spreadArray(to, from, pack) {
        if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
            if (ar || !(i in from)) {
                if (!ar) ar = Array.prototype.slice.call(from, 0, i);
                ar[i] = from[i];
            }
        }
        return to.concat(ar || Array.prototype.slice.call(from));
    }

    var version = "0.5.0";
    var author = "Tom Scott <tubbo@psychedeli.ca>";
    var license = "MIT";

    // Money in RCT2 is expressed in dimes, e.g. $3 is "30"
    var PRICE_BIN = 30;
    var PRICE_BENCH = 50;
    var PRICE_QUEUETV = 150;
    function Add(settings) {
        var paths = { unsloped: [], sloped: [] };
        var queues = { queue: [] };
        var useMoney = !park.getFlag("noMoney");
        var _loop_1 = function (y) {
            var _loop_2 = function (x) {
                var elements = map.getTile(x, y).elements;
                var surface = elements.filter(function (element) { return element.type === "surface"; })[0];
                var footpaths = elements.filter(function (element) { return element.type === "footpath"; });
                footpaths.forEach(function (path) {
                    if (!canBuildAdditionOnPath(surface, path)) {
                        if (!canBuildAdditionOnQueue(surface, path)) {
                            return;
                        }
                        else {
                            queues.queue.push({ path: path, x: x, y: y });
                            return;
                        }
                    }
                    if (conflictsWithExistingAddition(path, settings)) {
                        return;
                    }
                    if ((path === null || path === void 0 ? void 0 : path.slopeDirection) === null) {
                        paths.unsloped.push({ path: path, x: x, y: y });
                    }
                    else {
                        paths.sloped.push({ path: path, x: x, y: y });
                    }
                });
            };
            for (var x = 0; x < map.size.x; x++) {
                _loop_2(x);
            }
        };
        // Iterate every tile in the map
        for (var y = 0; y < map.size.y; y++) {
            _loop_1(y);
        }
        // Build benches and bins on unsloped paths
        paths.unsloped.forEach(function (_a) {
            var path = _a.path, x = _a.x, y = _a.y;
            var bench = settings.bench, bin = settings.bin;
            var _b = findAdditionAndPrice(bench, bin, x, y), addition = _b[0], price = _b[1];
            var cash = park.cash - price;
            if (useMoney && cash >= 0) {
                ensureHasAddition(path, addition, price);
            }
            else if (!useMoney) {
                ensureHasAddition(path, addition, 0);
            }
            else {
                throw new Error("Insufficient funds.");
            }
        });
        // Build bins on sloped paths
        paths.sloped.forEach(function (_a) {
            var path = _a.path, x = _a.x, y = _a.y;
            var cash = park.cash - PRICE_BIN;
            var affordable = cash >= 0;
            var buildBinsOnAllSlopedPaths = settings.buildBinsOnAllSlopedPaths;
            var evenTile = x % 2 === y % 2;
            var buildOnSlopedPath = buildBinsOnAllSlopedPaths || evenTile;
            if (useMoney && buildOnSlopedPath && affordable) {
                ensureHasAddition(path, settings.bin, PRICE_BIN);
            }
            else if (!useMoney && buildOnSlopedPath) {
                ensureHasAddition(path, settings.bin, 0);
            }
        });
        // Build queue tvs on queue lines
        queues.queue.forEach(function (_a) {
            var path = _a.path;
            var queuetv = settings.queuetv;
            var _b = [queuetv, PRICE_QUEUETV], addition = _b[0], price = _b[1];
            var cash = park.cash - price;
            if (useMoney && cash >= 0) {
                ensureHasAddition(path, addition, price);
            }
            else if (!useMoney) {
                ensureHasAddition(path, addition, 0);
            }
            else {
                throw new Error("Insufficient funds.");
            }
        });
        return paths;
    }
    function conflictsWithExistingAddition(path, settings) {
        return (path.addition !== null && settings.preserveOtherAdditions);
    }
    function ensureHasAddition(path, addition, price) {
        if (path.addition !== addition || path.isAdditionBroken) {
            path.addition = addition;
            path.isAdditionBroken = false;
            path.isAdditionGhost = false;
            park.cash -= price;
        }
    }
    function findAdditionAndPrice(bench, bin, x, y) {
        if (x % 2 === y % 2) {
            return [bench, PRICE_BENCH];
        }
        else {
            return [bin, PRICE_BIN];
        }
    }
    function canBuildAdditionOnPath(surface, path) {
        if (!surface || !path) {
            return false;
        }
        if (path.isQueue) {
            return false;
        }
        if (surface.hasOwnership) {
            return true;
        }
        // Only allowed to build underground or elevated on land with construction rights
        if (surface.hasConstructionRights && surface.baseHeight !== path.baseHeight) {
            return true;
        }
        return false;
    }
    function canBuildAdditionOnQueue(surface, path) {
        if (!surface || !path) {
            return false;
        }
        if (!path.isQueue) {
            return false;
        }
        if (surface.hasOwnership) {
            return true;
        }
        // Only allowed to build underground or elevated on land with construction rights
        if (surface.hasConstructionRights && surface.baseHeight !== path.baseHeight) {
            return true;
        }
        return false;
    }

    var BENCH = "Benchwarmer.Bench";
    var BIN = "Benchwarmer.Bin";
    var QUEUETV = "Benchwarmer.QueueTV";
    var BUILD = "Benchwarmer.BuildOnAllSlopedFootpaths";
    var PRESERVE = "Benchwarmer.PreserveOtherAdditions";
    var Settings = /** @class */ (function () {
        function Settings(all) {
            this.benches = all.filter(function (a) { return a.identifier.includes("bench"); });
            this.bins = all.filter(function (a) { return a.identifier.includes("litter"); });
            this.queuetvs = all.filter(function (a) { return a.identifier.includes("qtv"); });
        }
        Object.defineProperty(Settings.prototype, "bench", {
            get: function () {
                return this.benches[this.selections.bench].index;
            },
            set: function (index) {
                context.sharedStorage.set(BENCH, index);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Settings.prototype, "bin", {
            get: function () {
                return this.bins[this.selections.bin].index;
            },
            set: function (index) {
                context.sharedStorage.set(BIN, index);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Settings.prototype, "queuetv", {
            get: function () {
                return this.queuetvs[this.selections.queuetv].index;
            },
            set: function (index) {
                context.sharedStorage.set(QUEUETV, index);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Settings.prototype, "selections", {
            get: function () {
                var bench = context.sharedStorage.get(BENCH, 0);
                var bin = context.sharedStorage.get(BIN, 0);
                var queuetv = context.sharedStorage.get(QUEUETV, 0);
                return { bench: bench, bin: bin, queuetv: queuetv };
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Settings.prototype, "buildBinsOnAllSlopedPaths", {
            get: function () {
                return context.sharedStorage.get(BUILD, false);
            },
            set: function (value) {
                context.sharedStorage.set(BUILD, value);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Settings.prototype, "preserveOtherAdditions", {
            get: function () {
                return context.sharedStorage.get(PRESERVE, true);
            },
            set: function (value) {
                context.sharedStorage.set(PRESERVE, value);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Settings.prototype, "configured", {
            get: function () {
                return this.bench !== null && this.bin !== null;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Settings.prototype, "queueTVConfigured", {
            get: function () {
                return this.queuetv !== null;
            },
            enumerable: false,
            configurable: true
        });
        return Settings;
    }());

    var LABEL_X = 10;
    var INPUT_X = 70;
    var y = 0;
    function Document() {
        var widgets = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            widgets[_i] = arguments[_i];
        }
        y = 0;
        return widgets;
    }
    function Dropdown(text, choices, selectedIndex, onChange) {
        var items = choices.map(function (b) { return b.name + " " + b.identifier; });
        y += 20;
        return [
            {
                type: "label",
                x: LABEL_X,
                y: y,
                width: 60,
                height: 10,
                text: text,
            },
            {
                type: "dropdown",
                x: INPUT_X,
                y: y,
                width: 200,
                height: 10,
                items: items,
                selectedIndex: selectedIndex,
                onChange: onChange,
            },
        ];
    }
    function Checkbox(text, isChecked, onChange) {
        y += 15;
        return {
            type: "checkbox",
            x: LABEL_X,
            y: y,
            width: 200,
            height: 15,
            isChecked: isChecked,
            text: text,
            onChange: onChange,
        };
    }
    function Button(text, onClick) {
        y += 20;
        return {
            type: "button",
            text: text,
            x: 10,
            y: y,
            width: 50,
            height: 20,
            onClick: onClick,
        };
    }

    var name = "Benchwarmer";
    function main() {
        ui.registerMenuItem(name, function () {
            var additions = context.getAllObjects("footpath_addition");
            var settings = new Settings(additions);
            var window = ui.openWindow({
                title: name,
                id: 1,
                classification: name,
                width: 300,
                height: 140,
                widgets: Document.apply(void 0, __spreadArray(__spreadArray(__spreadArray(__spreadArray([], Dropdown("Bench:", settings.benches, settings.selections.bench, function (index) {
                    settings.bench = index;
                })), Dropdown("Bin:", settings.bins, settings.selections.bin, function (index) {
                    settings.bin = index;
                })), Dropdown("Queue TV:", settings.queuetvs, settings.selections.queuetv, function (index) {
                    settings.queuetv = index;
                })), [Checkbox("Build bins on all sloped footpaths", settings.buildBinsOnAllSlopedPaths, function (checked) {
                        settings.buildBinsOnAllSlopedPaths = checked;
                    }),
                    Checkbox("Preserve other additions (e.g. lamps)", settings.preserveOtherAdditions, function (checked) {
                        settings.preserveOtherAdditions = checked;
                    }),
                    Button("Add", function () {
                        if (settings.configured) {
                            try {
                                Add(settings);
                            }
                            catch (e) {
                                ui.showError("Error Building Benches/Bins", e.message);
                            }
                        }
                        window.close();
                    })]))
            });
        });
    }
    registerPlugin({
        name: name,
        version: version,
        licence: license,
        authors: [author],
        type: "local",
        main: main,
    });

}());
